<?php
/*
* File:     imap.php
* Category: config
* Author:   M. Goldenbaum
* Created:  24.09.16 22:36
* Updated:  -
*
* Description:
*  -
*/

use Webklex\PHPIMAP\IMAP;

return [

	/*
	|--------------------------------------------------------------------------
	| IMAP default account
	|--------------------------------------------------------------------------
	|
	| The default account identifier. It will be used as default for any missing account parameters.
	| If however the default account is missing a parameter the package default will be used.
	| Set to 'false' [boolean] to disable this functionality.
	|
	*/
	'default' => 'default',

	/*
	|--------------------------------------------------------------------------
	| Default date format
	|--------------------------------------------------------------------------
	|
	| The default date format is used to convert any given Carbon::class object into a valid date string.
	| These are currently known working formats: "d-M-Y", "d-M-y", "d M y"
	|
	*/
	'date_format' => 'd-M-Y',

	/*
	|--------------------------------------------------------------------------
	| Available IMAP accounts
	|--------------------------------------------------------------------------
	|
	| Please list all IMAP accounts which you are planning to use within the
	| array below.
	|
	*/
	'accounts' => [

		'libriciel' => [// account identifier
			'host'  => 'url_mail',
			'port'  => 993,
			'protocol'  => 'imap', //might also use imap, [pop3 or nntp (untested)]
			'encryption'    => 'ssl', // Supported: false, 'ssl', 'tls'
			'validate_cert' => true,
			'username' => 'adresse_mail_to_check',
			'password' => 'AccessToken',
			'authentication' => 'imap',
		],

		'gmail' => [ // account identifier
			'host' => 'imap.gmail.com',
			'port' => 993,
			'protocol'  => 'imap',
			'encryption' => 'ssl',
			'validate_cert' => true,
			'username' => 'gmail_email',
			'password' => 'AccessToken',
			'authentication' => 'imap',
		],


		'outlook' => [ // account identifier
			'host' => 'outlook.office365.com',
			'port' => 993,
			'protocol' => 'imap',
			'encryption' => false,
			'validate_cert' => true,
			'username' => 'email_outlook',
			'password' => 'AccessToken',
			'authentication' => 'oauth',
		]

	],

	/*
	|--------------------------------------------------------------------------
	| Available IMAP options
	|--------------------------------------------------------------------------
	|
	| Available php imap config parameters are listed below
	|   -Delimiter (optional):
	|       This option is only used when calling $oClient->
	|       You can use any supported char such as ".", "/", (...)
	|   -Fetch option:
	|       IMAP::FT_UID  - Message marked as read by fetching the body message
	|       IMAP::FT_PEEK - Fetch the message without setting the "seen" flag
	|   -Body download option
	|       Default TRUE
	|   -Flag download option
	|       Default TRUE
	|   -Message key identifier option
	|       You can choose between 'id', 'number' or 'list'
	|       'id'     - Use the MessageID as array key (default, might cause hickups with yahoo mail)
	|       'number' - Use the message number as array key (isn't always unique and can cause some interesting behavior)
	|       'list'   - Use the message list number as array key (incrementing integer (does not always start at 0 or 1)
	|   -Fetch order
	|       'asc'  - Order all messages ascending (probably results in oldest first)
	|       'desc' - Order all messages descending (probably results in newest first)
	|   -Common folders
	|       Default folder locations and paths assumed if none is provided
	|   -Open IMAP options:
	|       DISABLE_AUTHENTICATOR - Disable authentication properties.
	|                               Use 'GSSAPI' if you encounter the following
	|                               error: "Kerberos error: No credentials cache
	|                               file found (try running kinit) (...)"
	|                               or ['GSSAPI','PLAIN'] if you are using outlook mail
	|   -Decoder options (currently only the message subject and attachment name decoder can be set)
	|       'utf-8' - Uses imap_utf8($string) to decode a string
	|       'mimeheader' - Uses mb_decode_mimeheader($string) to decode a string
	|
	*/
	'options' => [
		'delimiter' => '/',
		'fetch' => 1,
		'fetch_body' => true,
		'fetch_flags' => true,
		'message_key' => 'list',
		'fetch_order' => 'asc',
		'common_folders' => [
			"root" => "INBOX",
			"junk" => "INBOX/Junk",
			"draft" => "INBOX/Drafts",
			"sent" => "INBOX/Sent",
			"trash" => "INBOX/Trash",
		],
		'open' => [
			// 'DISABLE_AUTHENTICATOR' => 'GSSAPI'
		],
		'decoder' => [
			'message' => 'utf-8', // mimeheader
			'attachment' => 'utf-8' // mimeheader
		]
	],

	/*
	|--------------------------------------------------------------------------
	| Available events
	|--------------------------------------------------------------------------
	|
	 */
	'events' => [
		"message" => [
			'new' => \Webklex\PHPIMAP\Events\MessageNewEvent::class,
			'moved' => \Webklex\PHPIMAP\Events\MessageMovedEvent::class,
			'copied' => \Webklex\PHPIMAP\Events\MessageCopiedEvent::class,
			'deleted' => \Webklex\PHPIMAP\Events\MessageDeletedEvent::class,
			'restored' => \Webklex\PHPIMAP\Events\MessageRestoredEvent::class,
		],
		"folder" => [
			'new' => \Webklex\PHPIMAP\Events\FolderNewEvent::class,
			'moved' => \Webklex\PHPIMAP\Events\FolderMovedEvent::class,
			'deleted' => \Webklex\PHPIMAP\Events\FolderDeletedEvent::class,
		],
		"flag" => [
			'new' => \Webklex\PHPIMAP\Events\FlagNewEvent::class,
			'deleted' => \Webklex\PHPIMAP\Events\FlagDeletedEvent::class,
		],
	],

	/*
	|--------------------------------------------------------------------------
	| Available masking options
	|--------------------------------------------------------------------------
	|
	| By using your own custom masks you can implement your own methods for
	| a better and faster access and less code to write.
	|
	| Checkout the two examples custom_attachment_mask and custom_message_mask
	| for a quick start.
	|
	| The provided masks below are used as the default masks.
	 */
	'masks' => [
		'message' => \Webklex\PHPIMAP\Support\Masks\MessageMask::class,
		'attachment' => \Webklex\PHPIMAP\Support\Masks\AttachmentMask::class
	]
];
