<?php
/**
 * A CakePHP shell to create a fixture using the values of any given database table so one does not have to type it up manually.
 *
 * Copyright 2008, Debuggable, Ltd.
 * Hibiskusweg 26c
 * 13089 Berlin, Germany
 *
 * Licensed under The MIT License
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright Copyright 2008, Debuggable, Ltd.
 * @version 1.0
 * @author Felix Geisendörfer <felix@debuggable.com>, Tim Koschützki <tim@debuggable.com>
 * @license http://www.opensource.org/licenses/mit-license.php The MIT License
 */
App::import(array('Model', 'AppModel', 'File'));
App::import(array('Model', 'Courrier', 'File'));


App::uses('Model', 'AppModel');
App::uses('Folder', 'Utility');
App::uses('File', 'Utility');
App::uses('XShell', 'Console/Command');

App::uses('DataAclNode', 'DataAcl.Model');
class FixturizeShell extends XShell{
	function main() {
		if ($this->args && $this->args[0] == '?') {
			return $this->out('Usage: ./cake fixturize <table> [-force] [-reindex]');
		}
		$options = array(
			'force' => false,
			'reindex' => false,
			'all' => false,
		);
		foreach ($this->params as $key => $val) {
			foreach ($options as $name => $option) {
				if (isset($this->params[$name]) || isset($this->params['-'.$name]) || isset($this->params[$name{0}])) {
					$options[$name] = true;
				}
			}
		}

		if ($options['all']) {
			$db = ConnectionManager::getDataSource('test');
			$this->args = $db->listSources();
		}

		if (empty($this->args)) {
			return $this->err('Usage: ./cake fixturize <table>');
		}

		foreach ($this->args as $table) {
			$name = Inflector::classify($table);

//			$Model = new AppModel(array(
//				'name' => $name,
//				'table' => $table,
//			));

			$Model = ClassRegistry::init($name);
			$Model->setDataSource('test');

			$file = sprintf('%sTest/Fixture/%s_fixture.php', APP, Inflector::underscore($name));
			$File = new File($file);
			if ($File->exists() && !$options['force']) {
				$this->err(sprintf('File %s already exists, use --force option.', $file));
				continue;
			}
			$records = $Model->find('all', array('contain' => false, 'recursive' => -1));

			$out = array();
			$out[] = '<?php';
			$out[] = '';
			$out[] = sprintf('class %sFixture extends CakeTestFixture {', $name);
			$out[] = sprintf('	var $name = \'%s\';', $name);
			$out[] = '	var $records = array(';
			foreach ($records as $record) {
				$out[] = '		array(';
				if ($options['reindex']) {
					foreach (array('old_id', 'vendor_id') as $field) {
						if ($Model->hasField($field)) {
							$record[$name][$field] = $record[$name]['id'];
							break;
						}
					}
					$record[$name]['id'] = String::uuid();
				}
				foreach ($record[$name] as $field => $val) {
					if (is_null($val)) {
						$out[] = sprintf('			\'%s\' => null,', addcslashes($field, "'"));
					} else if (is_numeric($val)) {
						$out[] = sprintf('			\'%s\' => %s,', addcslashes($field, "'"), $val);
					} else {
						$out[] = sprintf('			\'%s\' => \'%s\',', addcslashes($field, "'"), addcslashes($val, "'"));
					}
				}
				$out[] = '		),';
			}
			$out[] = '	);';
			$out[] = '}';
			$out[] = '';
			$out[] = '?>';

			$File->write(join("\n", $out));
			$this->out(sprintf('-> Create %sFixture with %d records (%d bytes) in "%s"', $name, count($records), $File->size(), $file));
		}
	}
}

?>
